//---------------------------------------------------------------------------
// Flight Simulator X - Shader Effect Files
// Copyright (c) 2006, Microsoft Corporation
//---------------------------------------------------------------------------
//
// Just copy the base texture
//

#include <Common.fxh>
#include <MaterialDecl.fxh>
#include <D3D9Texture.fxh>
#include <FuncLibrary.fxh>

const bool EffectDeclaration
<
    string Name      = "Black";
    string Class     = "Basic";
    string Fallback  = "Assert";
> = true;

// Fog
//
float   g_fFogDensity           : FOG_DENSITY;
float4  g_cFog                  : FOG_COLOR;

struct VS_OUTPUT
{
    float4 vPos    : POSITION;
    float  fFog    : FOG;
};

struct VS_INPUT
{
    float4 vPos    : POSITION;
};

VS_OUTPUT
VS_GetBlack(const VS_INPUT v)
{
    VS_OUTPUT o;

    o.vPos = mul(vPos, mFullProj);

    o.fFog = FogVS(v.vPos, mWorld, vEyePoint);

    return o;
}

float4
PS_GetBlack(VS_OUTPUT In) : COLOR
{
    float4 Color = (0,0,0,0);
    return float4(FogPS(Color.xyz, In.fFog, g_fFogDensity, g_cFog), Color.a);
}

technique T0
<
    int    Performance = EffectPerfLevel_MaxShader30;
>
{
    pass P0
    {
        VertexShader             = compile vs_3_0 VS_GetBlack();
        PixelShader              = compile ps_3_0 PS_GetBlack();
        FogEnable                = False;
        AlphaBlendEnable         = False;
        SrcBlend                 = One;
        DestBlend                = Zero;
	NormalizeNormals         = True;
	ZWriteEnable             = True;
	ZEnable                  = True;
	ZFunc                    = LessEqual;
	SeparateAlphaBlendEnable = False;
	AlphaTestEnable          = False;
	StencilEnable            = False;
	SpecularEnable           = False;
	ColorWriteEnable         = Red | Green | Blue;
	CullMode                 = CCW;
    }
}

